//  RW Mailstop
// ==============
// ______
// -Part of the RW Foundations suite of libraries

// Filename: ./include/dollhouse/net/mailstop_util.hpp
// -Declarations of some baseline RW Mailstop utility functions

#pragma once

///////
// Dolldrop/Dollnet is meant for network-enabled RW projects only
#if __has_include(<curl_easy.h>)

#include "incls_all.hpp"

namespace rw::dollhouse::net {

constexpr auto droplog{"./dolldrops/dropmod.json"};  // dolldrops log's filename

//------------------------------------------------------------------------------
/** A simple data struct to help with local @c Dolldrop 's log entries
 */
struct Dropdata {
  string date;
  string name;
  string uri;
};

//------------------------------------------------------------------------------
/** Assign to a dolldrop data struct from a JSON object
 * @param[in] j
 * @param[out] d
 */
void from_json(Json const& j, Dropdata& d);

//------------------------------------------------------------------------------
/** Assign to a JSON object from a dolldrop data struct
 * @param[out] j
 * @param[in] d
 */
void to_json(Json& j, Dropdata const& d);

//------------------------------------------------------------------------------
/** Initialize a @a tagfile file
 * @param tagfile_nm
 * @note: initial tag number defaults to @a '000', so @a '001' will be the first
 * (that is, the @a next) tag number delivered by @c filetag() whenever this
 * function call is required
 */
void mk_tagfile(std::string const& file_nm);

//------------------------------------------------------------------------------
/** File-based scheme for doling out incremental numbers and (optionally) a
 * specified file-extension, provided together as a single tag string
 * @return the (next) file tag
 * @note: intended to serve as a filename suffix for differentiating subsequent
 * @c Dolldrop files
 * @note: this reads the count number from 'tagfile', then return the @a next
 * number filetag (also writing this number out to tagfile for next cycle)
 */
auto filetag() -> std::string;

//------------------------------------------------------------------------------
/** Read a dolldrop header stream and locate it's @a field_nm tag, if present
 * @param[in] ss
 * @param[in] field_nm
 * @return the header's @a field_nm data, or empty string
 */
auto dropfield(std::stringstream& ss, std::string const& field_nm)
    -> std::string;

//------------------------------------------------------------------------------
/** Compare the dolldrop's header's current & stored 'last-modified' tags
 * @param uri
 * @param hdr
 * @return a @c std::tuple for use by dolldrop caller
 */
auto chk_droptimes(std::string const& uri, std::stringstream& hdr)
    -> std::tuple<bool, std::string>;

//------------------------------------------------------------------------------
/** Set a droplog record entry's specified @a tag with @a tagdat value
 * @param uri
 * @param tag
 * @param tagdat
 * @note: @a uri parm is the search key to find specific record in log
 */
void set_tag(string const& uri, string const& tag, string const& tagdat);
/** Simple wrapper
 * @param uri
 * @param datedat
 * @note: see @c set_tag()
 */
void set_date(string const& uri, string const& datedat);
/** Simple wrapper
 * @param uri
 * @param namedat
 * @note: see @c set_tag()
 */
void set_name(string const& uri, string const& namedat);
/** Simple wrapper
 * @param uri
 * @param uridat
 * @note: see @c set_tag()
 */
void set_uri(string const& uri, string const& uridat);

//------------------------------------------------------------------------------
/** Return a droplog record entry's specified @a tag value
 * @param uri
 * @param tag
 * @return
 * @note: @a uri parm is the search key to find specific record in log
 */
auto get_tag(string const& uri, string const& tag) -> std::string;
/** Simple wrapper
 * @param uri
 * @return
 * @note: see @c get_tag()
 */
auto get_date(string const& uri) -> std::string;
/** Simple wrapper
 * @param uri
 * @return
 * @note: see @c get_tag()
 */
auto get_name(string const& uri) -> std::string;

}  // namespace rw::dollhouse::net

#endif  // __has_include(<curl_easy.h>)

//    --=^=--

// "______"

// rw_dollhouse_lib.v220205
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
