//   RW Dollnet
// ==============
// External, Inter-Networking Library
// -Part of the RW Foundations suite of libraries

// Filename: ./src/dollhouse/net/dollnet.cpp
// -Definitions of the baseline RW Dollnet sub-library

///////
// Dolldrop/Dollnet is meant for network-enabled RW projects only
#if __has_include(<curl_easy.h>)

#include "incls_all.hpp"

#include "curl_easy.h"
#include "curl_form.h"

#include "dollnet_util.cpp"
#include "mailstop_util.cpp"

// DESIGN: during this prototyping phase, code-envy is acceptable. however, for
// the production code abstract common behaviours into modular functions.
// -DRY principle

namespace rw::dollhouse::net {

using curl::curl_easy;
using curl::curl_easy_exception;
using curl::curl_ios;

//------------------------------------------------------------------------------
Dolldrop_b::Mailstop::Dollnet::Dollnet(Hip const&         tag,
                                       std::string const& handle)
    : RW_slave{{Type::Dolldrop}, handle} {
  stamp("Dollnet()");

  bind_master(tag);
}

//------------------------------------------------------------------------------
auto Dolldrop_b::Mailstop::Dollnet::dl_body_only(Hip const&         tag,
                                                 std::string const& uri)
    -> std::fstream {
  stamp(tag, "dl_body_only()");

  if (tag == master()) {  // function is meant only for slave's Master to use

    string path_to_file{""};
    //
    if (auto prevfile = get_name(uri); ! prevfile.empty()) {  // reuse dolldrop
      path_to_file = prevfile;

    } else {  // create a new dolldrop

      string const afile = "anons_file" + filetag();
      path_to_file       = "./dolldrops/" + afile;
    }
    //
    ofstream anons_file{};
    anons_file.open(path_to_file);

    curl_ios<ostream> writer{anons_file};
    curl_easy         easy{writer};

    easy.add<CURLOPT_USERAGENT>("Bumpmaster_v220205");
    easy.add<CURLOPT_XFERINFOFUNCTION>(prt_prog);
    easy.add<CURLOPT_NOPROGRESS>(0L);
    easy.add<CURLOPT_FOLLOWLOCATION>(1L);
    easy.add<CURLOPT_URL>(uri.c_str());

    try {  //

      start = Call_clk::now();
      easy.perform();
      cout << "\n\n   done\n";

    } catch (curl_easy_exception const& e) {  //

      cerr << "\n\n -in Dollnet::dl_body_only()::try{} :\n" << e.what() << '\n';
      e.print_traceback();

      // TEMP: till we put in place more-robust connection mgmt, we'll just exit
      anons_file.close();
      std::exit(EXIT_FAILURE);
    }

    anons_file.close();
    set_name(uri, path_to_file);

    return fstream{path_to_file};  // return handle to now-saved file

  } else
    only_master_can(master(), tag, t(), "Dollnet::dl_body_only()");

  ///
  return fstream{};  // shouldn't reach here
}

//------------------------------------------------------------------------------
auto Dolldrop_b::Mailstop::Dollnet::dl_w_header(Hip const&         tag,
                                                std::string const& uri)
    -> std::stringstream {
  stamp(tag, "dl_w_header()");

  if (tag == master()) {  // function is meant only for slave's Master to use

    // append a doled-out tag to dolldrop's name (next file number increment)
    string const afile = "anons_file" + filetag();

    // extend name out to project's dolldrops filepath
    string const path_to_file = "./dolldrops/" + afile;

    // create a standard output file
    ofstream anons_file{};

    // file will be written out in the project's ./dolldrops/ directory
    anons_file.open(path_to_file);

    // the "writer" stream will put the DL body's contents into "anons_file"
    curl_ios<ostream> writer{anons_file};

    // pass this stream to the Easy object's constructor for writing to disk
    curl_easy easy{writer};

    // we'll return this header data stream at the end of this code block
    stringstream header_dat{};

    // the "header" stream will put the DL header's contents into "header_dat"
    curl_ios<stringstream> header{header_dat};

    // we'll use curlcpp's default write function for "header"
    easy.add<CURLOPT_WRITEFUNCTION>(header.get_function());

    // specify which stream (ie, header data) to use for "header" 's contents
    easy.add<CURLOPT_HEADERDATA>(header.get_stream());

    // r/n we're simply known as 'Bumpmaster' to the rest of the world
    easy.add<CURLOPT_USERAGENT>("Bumpmaster_v220205");

    // we'll output our DL progress to the console
    easy.add<CURLOPT_XFERINFOFUNCTION>(prt_prog);

    // yes, do go ahead and track our DL progress
    easy.add<CURLOPT_NOPROGRESS>(0L);

    // yes, follow locations
    easy.add<CURLOPT_FOLLOWLOCATION>(1L);

    // and here's where we're going
    easy.add<CURLOPT_URL>(uri.c_str());

    try {  // can catch exceptions for this curlcpp Easy object (which is great)

      // start the timer,
      start = Call_clk::now();

      // and let's go!
      easy.perform();

      // if we're here, the DL connection has apparently finished successfully
      cout << "\n\n   done\n\n";

    } catch (curl_easy_exception const& e) {  // oh noes!11  did it broke?

      // prints the last error
      cerr << "\n\n -in Dollnet::dl_w_header()::try{} :\n" << e.what() << '\n';

      // prints the entire error stack
      e.print_traceback();

      // TEMP: till we put in place more-robust connection mgmt, we'll just exit
      anons_file.close();
      std::exit(EXIT_FAILURE);
    }

    // _ALWAYS_ close output files Anon!  :^)
    anons_file.close();

    // preserve the dolldrop's pathname out to log file
    set_name(uri, path_to_file);

    // make sure everything's written out for header data
    header_dat << flush;

    // we'll send any DL header data we found back to caller
    return header_dat;

  } else
    only_master_can(master(), tag, t(), "Dollnet::dl_w_header()");

  ///
  return stringstream{};  // shouldn't reach here
}

//------------------------------------------------------------------------------
auto Dolldrop_b::Mailstop::Dollnet::dl_hdr_only(const Hip&         tag,
                                                const std::string& uri)
    -> std::stringstream {
  stamp(tag, "dl_hdr_only()");

  if (tag == master()) {  // function is meant only for slave's Master to use

    stringstream           header_dat{};
    curl_ios<stringstream> header{header_dat};
    curl_easy              easy{header};

    easy.add<CURLOPT_HEADER>(1L);
    easy.add<CURLOPT_NOBODY>(1L);
    easy.add<CURLOPT_XFERINFOFUNCTION>(prt_time);  // just time (since no size)
    easy.add<CURLOPT_NOPROGRESS>(0L);
    easy.add<CURLOPT_USERAGENT>("Bumpmaster_v220205");
    easy.add<CURLOPT_FOLLOWLOCATION>(1L);
    easy.add<CURLOPT_URL>(uri.c_str());

    try {  //

      start = Call_clk::now();
      easy.perform();
      puts("");

    } catch (curl_easy_exception const& e) {  //

      cerr << "\n\n -in Dollnet::dl_hdr_only()::try{} :\n" << e.what() << '\n';
      e.print_traceback();

      // TEMP: till we put in place more-robust connection mgmt, we'll just exit
      std::exit(EXIT_FAILURE);
    }

    header_dat << flush;
    return header_dat;

  } else
    only_master_can(master(), tag, t(), "Dollnet::dl_hdr_only()");

  ///
  return stringstream{};  // shouldn't reach here
}

}  // namespace rw::dollhouse::net

#endif  // __has_include(<curl_easy.h>)

//    --=^=--

// "This apartment has ultra-high bandwidth illegal even to large corporations?"

// rw_dollhouse_lib.v220205
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
