//  RW Mailstop
// ==============
// ______
// -Part of the RW Foundations suite of libraries

// Filename: ./src/dollhouse/net/mailstop_util.cpp
// -Definitions of some baseline RW Mailstop utility functions

///////
// Dolldrop/Dollnet is meant for network-enabled RW projects only
#if __has_include(<curl_easy.h>)

#include "incls_all.hpp"

#include "../../../include/dollhouse/net/mailstop_util.hpp"

namespace rw::dollhouse::net {

//------------------------------------------------------------------------------

static bool log_chkd{false};  // flag for checking once that the droplog exists

//------------------------------------------------------------------------------
void from_json(Json const& j, Dropdata& d) {
  d.date = j["date"];
  d.name = j["name"];
  d.uri  = j["uri"];
}

//------------------------------------------------------------------------------
void to_json(Json& j, Dropdata const& d) {
  j = Json{{"date", d.date}, {"name", d.name}, {"uri", d.uri}};
}

//------------------------------------------------------------------------------
void mk_tagfile(std::string const& file_nm) {
  ofstream o_file{};
  o_file.open(file_nm);
  o_file << "000\n";
  o_file.close();
}

//------------------------------------------------------------------------------
auto filetag() -> std::string {
  string const file_nm{"./dolldrops/tagfile"};

  // attempt to open tagfile
  ifstream i_file{};
  i_file.open(file_nm);
  //
  if (! i_file) {  // create if needed
    mk_tagfile(file_nm);
    i_file.open(file_nm);
  }

  // capture tagfile's contents into vars
  int    cnt{0};
  string ext{""};
  i_file >> cnt >> ext;  // DESIGN: any validation req'd here ?
  i_file.close();

  // store incremented count back out to file for next dolldrop cycle
  ofstream o_file{};
  o_file.open(file_nm);
  o_file << setw(3) << setfill('0') << ++cnt << '\n'
         << ((ext != "") ? ext + '\n' : "");  // also write ext if present
  o_file.close();

  // create file tag from now-incremented tagfile data
  ostringstream tagss{};
  tagss << '_' << setw(3) << setfill('0') << cnt << ext;

  return tagss.str();
}

//------------------------------------------------------------------------------
auto dropfield(std::stringstream& ss, std::string const& field_nm)
    -> std::string {  //

  auto lcfn{field_nm};  // 1st mk cp for lowercase'd searches
  to_lower(lcfn);       //  "

  for (string line; getline(ss, line);) {
    to_lower(line);

    if (line.find(lcfn) != str_end) {  // we have the correct header tag
      // rm the tag text itself
      auto fld_dat = line.substr(lcfn.size(), str_end);

      // rm '\r' char, if present in the data
      if (auto r_it = fld_dat.find('\r'); r_it != str_end)
        fld_dat.erase(r_it, 1);

      return fld_dat;  // just the facts ma'am  :^)
    }
  }

  return string{};  // dropfield not found
}

//------------------------------------------------------------------------------
void mk_logfile(std::string const& file_nm) {
  ofstream o_file{};

  o_file.open(file_nm);
  o_file << "[]";  // insert a default empty JSON array
  o_file.close();
}

//------------------------------------------------------------------------------
void chk_logfile() {
  ifstream i_file{};

  i_file.open(droplog);  // attempt to open log file
  if (! i_file)          // create if needed
    mk_logfile(droplog);

  i_file.close();
}

//------------------------------------------------------------------------------
void set_tag(string const& uri, string const& tag, string const& tagdat) {  //

  if (! log_chkd) {
    chk_logfile();
    log_chkd = true;
  }

  // construct cp of droplog's data
  fstream ifs{droplog, ios::in};
  auto    j_out = fstream_2_json(ifs);

  auto rec_it = find_if(begin(j_out), end(j_out),  // have the record?
                        [&uri](Json const& j) { return uri == j["uri"]; });

  if (rec_it != end(j_out)) {  // found the record, assign the new tag data

    if (tag == "date")
      rec_it->at("date") = tagdat;
    else if (tag == "name")
      rec_it->at("name") = tagdat;
    else if (tag == "uri")
      rec_it->at("uri") = tagdat;

  } else {
    puts(" dropmod.json entry not found; adding new record...");

    Dropdata d{};
    d.uri = uri;
    j_out.emplace_back(d);  // constructs new JSON item in-place, via to_json()
  }

  ofstream ofs{droplog};
  ofs << j_out;
  ofs.close();
}
// wrappers-related
void set_date(string const& uri, string const& datedat) {
  set_tag(uri, "date", datedat);
}
void set_name(string const& uri, string const& namedat) {
  set_tag(uri, "name", namedat);
}
void set_uri(string const& uri, string const& uridat) {
  set_tag(uri, "uri", uridat);
}

//------------------------------------------------------------------------------
auto get_tag(string const& uri, string const& tag) -> std::string {  //

  string res{""};

  if (! log_chkd) {
    chk_logfile();
    log_chkd = true;
  }

  // construct cp of droplog's data
  fstream    ifs{droplog, ios::in};
  auto const j_in = fstream_2_json(ifs);

  auto rec_it = find_if(cbegin(j_in), cend(j_in),  // have the record?
                        [&](Json const& j) { return uri == j["uri"]; });

  if (rec_it != cend(j_in)) {  // found the record, retrieve the tag's value

    auto const rec = *rec_it;

    if (tag == "date")
      res = rec["date"];
    else if (tag == "name")
      res = rec["name"];

  } else {  // uri's record not found, so this call will end up adding a new one

    set_uri(uri, uri);
  }

  return res;
}
// wrappers-related
auto get_date(string const& uri) -> std::string {
  return get_tag(uri, "date");
}
auto get_name(string const& uri) -> std::string {
  return get_tag(uri, "name");
}

//------------------------------------------------------------------------------
auto chk_droptimes(std::string const& uri, std::stringstream& hdr)
    -> std::tuple<bool, std::string> {  //

  auto const& prevdate = get_date(uri);
  auto const& currdate = dropfield(hdr, "last-modified: ");

  bool have_new{false};
  if (prevdate != currdate) {  //

    // TEST:
    cout << "\n  dolldrop's previous dropmod :  \t'" << prevdate << "'\n"  //
         << "  dolldrop's current dropmod :   \t'" << currdate << "'\n\n";

    set_date(uri, currdate);
    have_new = true;

  } else  // TEST:
    cout << "\n  dolldrop's current dropmod :  '" << currdate << "'\n";

  auto const& name = get_name(uri);

  return make_tuple(have_new, name);
}

}  // namespace rw::dollhouse::net

#endif  // __has_include(<curl_easy.h>)

//    --=^=--

// "______"

// rw_dollhouse_lib.v220205
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
